/*   DOOROOS: util_kernelugd.c   */

/*
 * Copyright (C) 2015    iBOSN systems
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted completely.
 *
 */

/*
 * Module Name: util_kernelugd.c
 *
 * Purpose: Create the user application.
 *
 */

#include <types.h>
#include <thread.h>
#include <window.h>
#include <time.h>
#include <memory.h>
#include <fs.h>
#include <lib/stdio.h>
#include <lib/string.h>
#include <panic.h>

//#include <sound.h>
#include <dev/kbd.h>

#include <gl/sdi.h>
#include <moduleloader.h>
#include <resource.h>


// debug level info
#define DBG_LVL_UPGRADE     0

#define Kernel_UPGRADE 1



#ifndef WM_COPYFILECOUNT
#define WM_COPYFILECOUNT                      7777
#endif 

#define TIMER_ID_HMIEXT          (1<<1)

#define STC_WARNING_TEXT                    84
#define STC_REBOOT_TEXT                     85

#define PROGRBAR_FILECOPY                   IDC_PROGBAR_DEF(81)

//#define STC_PASSWORD                            1031



#define GUI_TITLE_X      10.0
#define GUI_TITLE_Y      0.0
#define GUI_TITLE_WIDTH  80.0
#define GUI_TITLE_HEIGHT 10.0


#define GUI_BUTTON1_X      5.0
#define GUI_BUTTON1_Y      20.0
#define GUI_BUTTON1_WIDTH  25.0
#define GUI_BUTTON1_HEIGHT 60.0

#define GUI_BUTTON2_X      35.0
#define GUI_BUTTON2_Y      20.0
#define GUI_BUTTON2_WIDTH  30.0
#define GUI_BUTTON2_HEIGHT 60.0

#define GUI_BUTTON3_X      70.0
#define GUI_BUTTON3_Y      20.0
#define GUI_BUTTON3_WIDTH  25.0
#define GUI_BUTTON3_HEIGHT 60.0

// color info

#define COLOR_BG				    LT_GRAY
#define COLOR_BUTTON				LT_GRAY
#define COLOR_TITLE_TEXT 			BLACK

STATIC struct __HMIEXT_SETTING__
{
	INT32 option;
    //
}HmiExtSetting;

static HWND hWndHmiExt;
static HWND hWndKernel;
static HWND hWndZip;
static HWND hWndCopy;


//STATIC int downloadPercent = 0; // 0 ~ 100%
STATIC int totalUpgradeFiles = 0;
STATIC int currentUpgradedFiles = 0;

STATIC UINT32 HmiExtWndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );

STATIC VOID Widget_DrawHmiExtBg( PGL pGL, WIDGET_PDRAW_BACKGROUND pDrawBackground );
STATIC VOID Widget_DrawHmiExtButton( PGL pGL, WIDGET_PDRAW_BUTTON pDrawButton );



STATIC GL               Gl;

// dialog proc info
STATIC WNDCLASS         HmiExtWndClass = { CS_DBLCLKS | CS_VREDRAW | CS_HREDRAW, (PWNDPROC)HmiExtWndProc };

// gui position info
STATIC WIDGET           HmiExtWndWidget[] =
{	
	// background
	{WIDGET_TYPE_BACKGROUND, 
        IDC_BK_RSVD, 
                WIDGET_RECT(  0, 0, 100, 100), 
                NULL, 
		        (PWIDDRAWPROC)Widget_DrawHmiExtBg, &Gl, 
                {{NULL, NULL}}},
	
	// start button
	{WIDGET_TYPE_PUSHBUTTON, 
		IDC_BUTTON_DEF(1),
			WIDGET_RECT( GUI_BUTTON1_X,  GUI_BUTTON1_Y,  GUI_BUTTON1_WIDTH,  GUI_BUTTON1_HEIGHT), NULL, 
			(PWIDDRAWPROC)Widget_DrawHmiExtButton, &Gl, 
			{{"Kernel", NULL, NULL }}},
					
    // abort button
	{WIDGET_TYPE_PUSHBUTTON, 
		IDC_BUTTON_DEF(2),
			WIDGET_RECT( GUI_BUTTON2_X,  GUI_BUTTON2_Y,  GUI_BUTTON2_WIDTH,  GUI_BUTTON2_HEIGHT), NULL, 
			(PWIDDRAWPROC)Widget_DrawHmiExtButton, &Gl, 
			{{"Zipped", NULL, NULL }}},

    // abort button
	{WIDGET_TYPE_PUSHBUTTON, 
		IDC_BUTTON_DEF(3),
			WIDGET_RECT( GUI_BUTTON2_X,  GUI_BUTTON2_Y,  GUI_BUTTON2_WIDTH,  GUI_BUTTON2_HEIGHT), NULL, 
			(PWIDDRAWPROC)Widget_DrawHmiExtButton, &Gl, 
			{{"Copy", NULL, NULL }}},


    {-1,}
};


STATIC HANDLE hTimerHmiExt = NULL;

STATIC UINT32 HmiExtWndProc(
  HWND hWnd,      // handle to window
  UINT uMsg,      // message identifier
  WPARAM wParam,  // first message parameter
  LPARAM lParam   // second message parameter
)
{
    switch( uMsg )
    {
    case WM_CREATE:
    {
		DBGLINE(DBG_LVL_UPGRADE, "hWnd = %x, %d \n", hWnd, hWnd);

        if( hTimerHmiExt == NULL)
		{
			DBGLINE(DBG_LVL_UPGRADE, "create timer \n");
            hTimerHmiExt = TimerCreate( hWnd, TIMER_ID_HMIEXT, 500, NULL );
        }
    }break;    

//    case WM_TIMER:{
//		//DBGLINE(DBG_LVL_UPGRADE, "WM_TIMER call \n");
//        if( wParam & TIMER_ID_HMIEXT){
//            
//			//DBGLINE(DBG_LVL_UPGRADE, "TIMER_ID_HMIEXT call \n");
//            BOOL bRet;
//            
//            if(hTimerHmiExt){
//                TimerStop(hTimerHmiExt);
//            }
//
//            /* automatically start the upgrade process */
//            if(HmiExtSetting.option == Kernel_UPGRADE)
//            {
//                bRet = StartUpgrade_Kernel_File(hWnd);
//                if(!bRet){
//                    //ShowWidget(hWnd, IDC_STATIC_DEF(STATIC_WARNING_TEXT), SW_SHOW);
//                    //UpdateWidget(hWnd, IDC_STATIC_DEF(STATIC_WARNING_TEXT));
//                    //Sleep(200);
//                    ShowWindow(hWnd, SW_HIDE);
//                }
//            }
//        }
//    }break;

    case WM_COMMAND:
    {
        switch( LOWORD(wParam) )
        {
            case IDC_BUTTON_DEF(1):
			{
                ShowWindow( hWndKernel, SW_SHOW );
                break;
            }
			
            case IDC_BUTTON_DEF(2):
			{	
                ShowWindow( hWndZip, SW_SHOW );
                break;
            }

            case IDC_BUTTON_DEF(3):
            {
                ShowWindow( hWndCopy, SW_SHOW );
                break;
            }
        }//switch end
    }break;

    case WM_VSCROLL:
    case WM_HSCROLL:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_HSCROLL wParam = %x\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_HSCROLL lParam = %x\n",__FUNCTION__, lParam );
        
		break;

    case WM_KEYDOWN:
    {
        HWND    hWndFocus;

        hWndFocus = GetFocusWindow(hWnd);

        if (hWndFocus != hWnd)
        {
            SendMessage( hWndFocus, uMsg, wParam, lParam );
            break;
        }

        break;
    }
    
    case WM_KEYUP:
    {
        HWND    hWndFocus;

        hWndFocus = GetFocusWindow(hWnd);
        if (hWndFocus != hWnd)
        {
            SendMessage( hWndFocus, uMsg, wParam, lParam );
            break;
        }

        break;
    }
    case WM_CHAR:
        break;
        
    case WM_MOUSEMOVE:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_MOUSEMOVE wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_MOUSEMOVE lParam = %X\n",__FUNCTION__, lParam );
    	break;

    case WM_LBUTTONDOWN:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_LBUTTONDOWN wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_LBUTTONDOWN lParam = %X\n",__FUNCTION__, lParam );
    	break;
    case WM_LBUTTONUP:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_LBUTTONUP wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_LBUTTONUP lParam = %X\n",__FUNCTION__, lParam );
    	break;
    case WM_LBUTTONDBLCLK:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_LBUTTONDCLK wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_LBUTTONDCLK lParam = %X\n",__FUNCTION__, lParam );
    	break;

    case WM_RBUTTONDOWN:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_RBUTTONDOWN wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_RBUTTONDOWN lParam = %X\n",__FUNCTION__, lParam );
    	break;
    case WM_RBUTTONUP:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_RBUTTONUP wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_RBUTTONUP lParam = %X\n",__FUNCTION__, lParam );
    	break;
    case WM_RBUTTONDBLCLK:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_RBUTTONDCLK wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_RBUTTONDCLK lParam = %X\n",__FUNCTION__, lParam );
    	break;

    case WM_MBUTTONDOWN:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_MBUTTONDOWN wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_MBUTTONDOWN lParam = %X\n",__FUNCTION__, lParam );
    	break;
    case WM_MBUTTONUP:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_MBUTTONUP wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_MBUTTONUP lParam = %X\n",__FUNCTION__, lParam );
    	break;
    case WM_MBUTTONDBLCLK:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_MBUTTONDBLCLK wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_MBUTTONDBLCLK lParam = %X\n",__FUNCTION__, lParam );
    	break;

    case WM_MOUSEWHEEL:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_MOUSEWHEEL wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_MOUSEWHEEL lParam = %X\n",__FUNCTION__, lParam );
    	break;

    case WM_SHOWWINDOW:
        DEBUGMSG(0,"%s  WM_SHOWWINDOW wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(0,"%s  WM_SHOWWINDOW lParam = %X\n",__FUNCTION__, lParam );
        break;

    case WM_ACTIVATE:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_ACTIVATE wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_ACTIVATE lParam = %X\n",__FUNCTION__, lParam );
        break;

    case WM_SETFOCUS:
        DEBUGMSG(0,"%s  WM_SETFOCUS wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(0,"%s  WM_SETFOCUS lParam = %X\n",__FUNCTION__, lParam );
        
        
        break;

    case WM_KILLFOCUS:
        DEBUGMSG(0,"%s  WM_KILLFOCUS wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(0,"%s  WM_KILLFOCUS lParam = %X\n",__FUNCTION__, lParam );
        
        break;

    case WM_DESTROY:
        DEBUGMSG(DBG_LVL_UPGRADE,"A %s  WM_DESTROY wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_DESTROY lParam = %X\n",__FUNCTION__, lParam );
                
        break;

    default:
        break;
    }

    return 0;   
}

VOID FontSet( PGL pGL, UINT32 FontSize )
{
	BOOL	bAntiAlias = TRUE;
	GL_FontSetStyle( pGL, TTF_STYLE_SIZE, &FontSize );
	GL_FontSetStyle( pGL, TTF_STYLE_ANTIALIAS, &bAntiAlias );
}

BOOL
WidgetSetPos( PWIDGET pWidget, UINT32 Id, UINT32 xPos, UINT32 yPos, UINT32 Width, UINT32 Height)
{
    for( ; pWidget->Type != -1 ; pWidget++ )
    {
        if( pWidget->Id == Id )
            goto FOUND_WIDGET_ID;
    }

    return FALSE;

	FOUND_WIDGET_ID:
	{
		UINT32 wOrg;
		UINT32 hOrg;
		
		wOrg = pWidget->Rect.Xmax - pWidget->Rect.Xmin;
		hOrg = pWidget->Rect.Ymax - pWidget->Rect.Ymin;
	
		if( xPos   != -1) pWidget->Rect.Xmin = xPos;
		if( yPos   != -1) pWidget->Rect.Ymin = yPos;
		if( Width  != -1) pWidget->Rect.Xmax = pWidget->Rect.Xmin + Width - 1;  
		else              pWidget->Rect.Xmax = pWidget->Rect.Xmin + wOrg;
		if( Height != -1) pWidget->Rect.Ymax = pWidget->Rect.Ymin + Height - 1; 
		else              pWidget->Rect.Ymax = pWidget->Rect.Ymin + hOrg;
	}

    return TRUE;
} 

STATIC VOID ResizeWidgets_HmiExt(UINT32 Xres, UINT32 Yres)
{
    WidgetSetPos( HmiExtWndWidget, IDC_BK_RSVD, 0, 0, Xres, Yres);

	WidgetSetPos( HmiExtWndWidget, IDC_BUTTON_DEF(1), 
        Xres * (GUI_BUTTON1_X / 100.0), 
        Yres * (GUI_BUTTON1_Y / 100.0),
        Xres*(GUI_BUTTON1_WIDTH  / 100.0), 
        Yres*(GUI_BUTTON1_HEIGHT / 100.0) );	

	WidgetSetPos( HmiExtWndWidget, IDC_BUTTON_DEF(2), 
        Xres * (GUI_BUTTON2_X / 100.0), 
        Yres * (GUI_BUTTON2_Y / 100.0),
        Xres*(GUI_BUTTON2_WIDTH  / 100.0), 
        Yres*(GUI_BUTTON2_HEIGHT / 100.0) );	

	WidgetSetPos( HmiExtWndWidget, IDC_BUTTON_DEF(3), 
        Xres * (GUI_BUTTON3_X / 100.0), 
        Yres * (GUI_BUTTON3_Y / 100.0),
        Xres*(GUI_BUTTON3_WIDTH  / 100.0), 
        Yres*(GUI_BUTTON3_HEIGHT / 100.0) );	

    return;
}

PUBLIC HWND CreateWindow_HmiExtWnd( HWND hParentWnd )
{
    DBGLINE(DBG_LVL_UPGRADE, "CreateWindow_HmiExtWnd start@_@\n");

    HWND    hWnd;
    PSDI    pSDI;

    GL_Init(&Gl);

    pSDI = GL_GetSDI(&Gl);

	DBGLINE(DBG_LVL_UPGRADE, "pSDI->Xres(%d) pSDI->Yres(%d) \n", pSDI->Xres, pSDI->Yres);
    ResizeWidgets_HmiExt( pSDI->Xres, pSDI->Yres );
	//hWnd = CreateWindow( WndOpen_WndWidget, 
	//				&WndOpen_WndClass, 
    //                  0|WS_VISIBLE | WS_COMPATDRAW, 
    //                  pSDI->Xres/4, 
	//				 pSDI->Yres/8, 
	//				 Width, Height,
    //                  NULL, NULL );

    hWndHmiExt = CreateWindow( HmiExtWndWidget, &HmiExtWndClass,
                                0|WS_VISIBLE, 
                                0, 0, pSDI->Xres, pSDI->Yres, NULL, NULL );

    hWndKernel = CreateWindow_KernelWnd( hWndHmiExt );
    hWndZip    = CreateWindow_ZipWnd( hWndHmiExt );
    hWndCopy   = CreateWindow_CopyWnd( hWndHmiExt );

	DBGLINE(DBG_LVL_UPGRADE, "CreateWindow_HmiExtWnd end\n");
    
    return hWndHmiExt;
}



STATIC VOID
Widget_DrawHmiExtBg( PGL pGL, WIDGET_PDRAW_BACKGROUND pDrawBackground )
//Widget_DrawImageBase( PGL pGL, WIDGET_PDRAW_STATIC pDrawImage)
{   
//DBGLINE(DBG_LVL_UPGRADE, "Widget_DrawHmiExtBg start \n"); 
    RECT   Rect;    
    Rect = pDrawBackground->Rect;
    UINT32  Div;

    GL_SetForeColor( pGL, COLOR_BG);
    GL_RectFill( pGL, &Rect );    

    /* draw title text */
    GL_SetBkMode( pGL, GL_TRANSPARENCY_TRANSPARENT);//GL_TRANSPARENCY_TRANSPARENT , GL_TRANSPARENCY_OPAQUE
    GL_FontSetAlign( pGL, FONT_VCENTER|FONT_CENTER);
    
    GL_SetForeColor( pGL, COLOR_TITLE_TEXT ); // set black

	// set font size
	PSDI    pSDI;
	pSDI = GL_GetSDI(pGL);

    // set position
    Rect.Xmin = pSDI->Xres * GUI_TITLE_X / 100;
    Rect.Ymin = pSDI->Xres * GUI_TITLE_Y / 100;
    Rect.Xmax = pSDI->Xres * GUI_TITLE_WIDTH / 100;
    Rect.Ymax = pSDI->Xres * GUI_TITLE_HEIGHT / 100;
    
	int nWeight = 6;
	int nValue = pSDI->Xres/100;
    FontSet( pGL, nValue * nWeight );

    GL_TextDraw( pGL, &Rect, "HMI external utility" );
 
    return;
}



STATIC VOID
Rect3DDraw(PGL pGL, PRECT pRect )
{
    UINT32      cColor;

    GL_LineDraw( pGL, pRect->Xmin, pRect->Ymin, pRect->Xmax, pRect->Ymin);
    GL_LineDraw( pGL, pRect->Xmin, pRect->Ymin, pRect->Xmin, pRect->Ymax);

    cColor = pGL->cFore;
    pGL->cFore = pGL->cBack;
    GL_LineDraw( pGL, pRect->Xmin, pRect->Ymax, pRect->Xmax, pRect->Ymax);
    GL_LineDraw( pGL, pRect->Xmax, pRect->Ymin, pRect->Xmax, pRect->Ymax);
    pGL->cFore = cColor;

    return;
} 

STATIC VOID
Widget_DrawHmiExtButton(PGL pGL, WIDGET_PDRAW_BUTTON pDrawButton )
{
	//DBGLINE(DBG_LVL_UPGRADE,"Widget_DrawHmiExtButton start \n");
    RECT        Rect;
    RECT        RectFont;
    UINT32      cUp, cDn;

	//RectOffset( &pDrawButton->Rect, X, Y );
    Rect = pDrawButton->Rect;
    Rect.Ymax += 1;

    RectFont = Rect;
    RectInset( &RectFont, 1, 1);

    GL_SetForeColor( pGL, COLOR_BUTTON );
    GL_RectFill( pGL, &Rect );

    if( pDrawButton->State & (1<<0) )
    {
        // INSET
        cUp = DK_GRAY;
        cDn = WHITE;
    }
    else
    {
        // OUTSET
        cUp = WHITE;
        cDn = DK_GRAY;
    }

    GL_SetForeColor( pGL, cUp);
    GL_SetBackColor( pGL, cDn);

    Rect3DDraw( pGL, &pDrawButton->Rect );

//DEBUGMSG(DBG_LVL_UPGRADE,"GT_DrawPushButton RectFont = %D\n",RectFont.Xmin);
//DEBUGMSG(DBG_LVL_UPGRADE,"GT_DrawPushButton RectFont = %D\n",RectFont.Ymin);
//DEBUGMSG(DBG_LVL_UPGRADE,"GT_DrawPushButton RectFont = %D\n",RectFont.Xmax);
//DEBUGMSG(DBG_LVL_UPGRADE,"GT_DrawPushButton RectFont = %D\n",RectFont.Ymax);

    // draw text
    if( pDrawButton->State & (1<<1) )
    {
        //disabled
        GL_SetForeColor( pGL, WT_GRAY );
    }
    else
    {
        GL_SetForeColor( pGL, BLACK );		
    }

    if( pDrawButton->State & (1<<0) )
    {
        RectFont.Xmin -= 2;
        RectFont.Ymin -= 2;
    }

    PSDI    pSDI;
	pSDI = GL_GetSDI(pGL);

	int nWeight = 6;
	int nValue = pSDI->Xres/100.0;
    FontSet( pGL, nWeight * nValue );

    GL_SetBackColor( pGL, LT_GRAY );
    GL_FontSetAlign( pGL, FONT_CENTER|FONT_VCENTER);
    GL_TextDraw( pGL, &RectFont, pDrawButton->pData );
	
	//DBGLINE(DBG_LVL_UPGRADE,"Widget_DrawHmiExtButton end\n");

    return;
} 

