/*   DOOROOS: file_log.c   */

/*
 * Copyright (C) 2014    Jung Byong-Oh
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted completely.
 *
 */

/*
 * Module Name: file_log.c
 *
 * Purpose: Create the user application.
 *
 */

#include <types.h>
#include <time.h>
#include <fs.h>

#include <panic.h>
#include <window.h>
#include <lib/string.h>
#include <lib/stdio.h>

#include "file_log.h"

//#include "micool_user_data.h"

#define		MAX_TEMPBUF		2048

STATIC BYTE			TempBuf[MAX_TEMPBUF];

#define _USE_TEMP_FILE_

#ifdef _USE_TEMP_FILE_
static    char tmp_dst_file_name[MAX_PATH];
#endif

#define DBG_LVL     1

PUBLIC INT32 CheckDstDir(const char* dst_dir_path)
{
    int i;
    for(i = 0; i < 10; i++){
        if(!FsChangeDir((PBYTE)dst_dir_path)){
            if(i > 5){
                return ERROR_CANNOT_CHANGE_DIR;    
            }
        }
    } 

    return ERROR_NONE;
}

PUBLIC INT32 MakeDstDir(const char* dst_dir_path)
{
    DBGLINE(DBG_LVL, "dst_dir_path=%s\n", dst_dir_path);
    char* tmp_dir;
    char tmp_dir_name[MAX_PATH];
    memset(tmp_dir_name, 0, MAX_PATH);
    
    int i;
    int j;
    j = 0;
    for(i = 0; i < strlen(dst_dir_path); i++){
        if(dst_dir_path[i] == '/'){
            DBGLINE(DBG_LVL, "i=%d\n", i);
            if(j<i){
                strncat(tmp_dir_name, dst_dir_path+j, i-j);
                DBGLINE(DBG_LVL, "tmp_dir_name=%s\n", tmp_dir_name);
                if(!FsChangeDir((PBYTE)tmp_dir_name)){
                    if(!FsMakeDir((PBYTE)tmp_dir_name)){
                        DBGLINE(DBG_LVL, "FsMakeDir (%s) fail !\n", tmp_dir_name);
                        return ERROR_CANNOT_MAKE_DIR;
                    }else{
                        DBGLINE(DBG_LVL, "FsMakeDir (%s) success !\n", tmp_dir_name);
                    }
                }else{
                    DBGLINE(DBG_LVL, "FsChangeDir (%s) success !\n", tmp_dir_name);
                }
            }
            
            tmp_dir = ((char*)dst_dir_path)+i;
            DBGLINE(DBG_LVL, "tmp_dir=%s\n", tmp_dir);
            
            j = i;
            
        }
    }

    if(!FsChangeDir((PBYTE)dst_dir_path)){
        if(!FsMakeDir((PBYTE)dst_dir_path)){
            DBGLINE(DBG_LVL, "FsMakeDir (%s) fail !\n", tmp_dir_name);
            return ERROR_CANNOT_MAKE_DIR;
        }else{
            DBGLINE(DBG_LVL, "FsMakeDir (%s) success !\n", dst_dir_path);
        }
    }else{
        DBGLINE(DBG_LVL, "FsChangeDir (%s) success !\n", dst_dir_path);
    }

    return ERROR_NONE;
}


PUBLIC INT32 AnyFileCopy(char* SrcFileName, char* DstDirPath)
{
    HANDLE  hFileDest;
    HANDLE  hFileSrc;

    char DstTempFileName[MAX_PATH];
    char DstFileName[MAX_PATH];

    memset(DstTempFileName, 0, MAX_PATH);
    memset(DstFileName, 0, MAX_PATH);

DBGLINE(DBG_LVL, "SrcFileName=%s\n", SrcFileName);
DBGLINE(DBG_LVL, "DstDirPath=%s\n", DstDirPath);
    hFileSrc = FsOpen( (PBYTE)SrcFileName );
DEBUGMSG(0,"(3)%s :hFileSrc = %x\n",__FUNCTION__, hFileSrc );
//DEBUGMSG(0,"%s :CopyPath.Name = %s\n",__FUNCTION__, CopyPath.Name );
    if( hFileSrc == NULL )
        return ERROR_CANNOT_OPEN_FILE;


    //int i;
    char* file_name = NULL;    

    file_name = strrchr(SrcFileName,'/');    
    DBGLINE(DBG_LVL, "file_name=%s\n", file_name);


    // now destination file create
    StrCpy( DstFileName, DstDirPath );
    StrCat( DstFileName, file_name );
DBGLINE(DBG_LVL, "DstFileName=%s\n", DstFileName);
    StrCpy( DstTempFileName, DstDirPath );
    StrCat( DstTempFileName, file_name );
    StrCat( DstTempFileName, ".tmp" );
DBGLINE(DBG_LVL, "DstTempFileName=%s\n", DstTempFileName);

    // open

    hFileDest = FsOpen( (PBYTE)DstTempFileName );
    if(hFileDest == NULL){
        BOOL bRet;
        bRet = FsCreate( (PBYTE)DstTempFileName );
    DEBUGMSG(0,"(5)%s :hFileDest = %x\n",__FUNCTION__, hFileDest );
        if( bRet == FALSE )
            return ERROR_CANNOT_CREATE_FILE;    

        hFileDest = FsOpen( (PBYTE)DstTempFileName );
DEBUGMSG(0,"(6)%s :hFileDest = %x\n",__FUNCTION__, hFileDest );
        if( hFileDest == NULL )
            return ERROR_CANNOT_OPEN_FILE;

    }

    // now copy
    do
    {
        INT32	nRead;

        nRead = FsRead( hFileSrc, TempBuf, MAX_TEMPBUF );
//DEBUGMSG(0,"%s : FsRead : nRead = %d\n",__FUNCTION__, nRead );
        if( nRead <= 0 )    // end
            break;

        do
        {
            nRead -= FsWrite( hFileDest, TempBuf, nRead );
//DEBUGMSG(0,"%s : FsWrite : nRead = %d\n",__FUNCTION__, nRead );
        }while(nRead > 0 );
    }while(1);

    if( hFileSrc ){
        FsClose( hFileSrc );
    }

    
        
    if( hFileDest){
        FsTruncate(hFileDest);
        FsClose( hFileDest );
    }
    

    
    INT32 ret;
    ret = FsRename((PBYTE)DstFileName, (PBYTE)DstTempFileName);
    DBGLINE(DBG_LVL, "ret=%d\n", ret);
    //ret = FsDelete((PBYTE)DstTempFileName);
//DBGLINE(DBG_LVL, "ret=%d\n", ret);
    DBGLINE(DBG_LVL, "Fin!!!\n");

    FsSync();

    return ERROR_NONE;
}

STATIC INT32 iCopyFileCount = 0;

// can get a process state by hWnd
INT32 CopyAll2( char* src_dir,  char* dst_dir,  HWND hWnd, BOOL* pbStop)
{
    DBGLINE(DBG_LVL, "CopyAll2 start \n");
    iCopyFileCount = 0;
    //DBGLINE(DBG_LVL, "\n");
    INT32 Ret;
    //DBGLINE(DBG_LVL, "\n");
    Ret = CopyAllWithDir2( src_dir,  dst_dir, hWnd, pbStop);
    //DBGLINE(DBG_LVL, "\n");
    if(Ret == ERROR_NONE){
        return iCopyFileCount;
    }
    //DBGLINE(DBG_LVL, "\n");
    return Ret;
}

INT32 CopyAll( char* src_dir,  char* dst_dir)
{
    iCopyFileCount = 0;
    INT32 Ret;

    Ret = CopyAllWithDir( src_dir,  dst_dir);
    if(Ret == ERROR_NONE){
        return iCopyFileCount;
    }

    return Ret;
}

INT32 GetCopyFileCount( VOID )
{
    return iCopyFileCount;
}

INT32 CopyAllWithDir2(char* src_dir,  char* dst_dir,  HWND hWnd, BOOL* pbStop)
{
    DBGLINE(DBG_LVL, "src_dir=%s\n", src_dir);
    DBGLINE(DBG_LVL, "dst_dir=%s\n", dst_dir);
    
    HANDLE hSrcFile;
    HANDLE hDstFile;

    INT32 Ret;
            
    char tmp_dst_path[MAX_PATH];
    char tmp_src_path[MAX_PATH];
    memset(tmp_dst_path, 0, MAX_PATH);
    memset(tmp_src_path, 0, MAX_PATH);
    strcpy(tmp_dst_path, dst_dir);
    strcpy(tmp_src_path, src_dir);
    
    
    FILEDATA FileData;
    FILEINFO FileInfo;

    DBGLINE(DBG_LVL, "pbStop(%x, %d) \n", pbStop, *pbStop);

    if(pbStop)
    {
        if(*pbStop == TRUE)
            return ERROR_CALLSTOP;
    }
    
    int nItem;
    for( nItem = 0; FsReadDir( (PBYTE)src_dir, &FileData, nItem) != 0 ; nItem++ )
    {
        DBGLINE(DBG_LVL, "FileData.Name=%s\n", FileData.Name);        
        DBGLINE(DBG_LVL, "pbStop(%x, %d) \n", pbStop, *pbStop);
        if(pbStop)
        {
            if(*pbStop == TRUE)
                return ERROR_CALLSTOP;
        }
        
        if(strcmp((char*)FileData.Name, ".") == 0){
            continue;
        }else if(strcmp((char*)FileData.Name, "..") == 0){
            continue;
        }
        
        if(FileData.Attribute & FILE_ATTRIBUTE_DIRECTORY){
            DBGLINE(DBG_LVL, "Dir: FileData.Name=%s\n", FileData.Name);
            memset(tmp_src_path, 0, MAX_PATH);
            strcpy(tmp_src_path, src_dir);

            memset(tmp_dst_path, 0, MAX_PATH);
            strcpy(tmp_dst_path, dst_dir);
            
            if(strlen(dst_dir)>0){                
                if(dst_dir[strlen(dst_dir)-1] != '/'){
                    strcat(tmp_dst_path, "/");
                }
                strcat(tmp_dst_path, (char*)FileData.Name);
            }else{
                return ERROR_UNKNOWN;
            }

            if(strlen(src_dir)>0){                
                if(src_dir[strlen(src_dir)-1] != '/'){
                    strcat(tmp_src_path, "/");
                }
                strcat(tmp_src_path, (char*)FileData.Name);
            }else{
                return ERROR_UNKNOWN;
            }

            
            DBGLINE(DBG_LVL, "Dir: tmp_dst_path=%s\n", tmp_dst_path);
            DBGLINE(DBG_LVL, "Dir: tmp_src_path=%s\n", tmp_src_path);
            
            if(FsGetFileInfo((PBYTE)tmp_dst_path, &FileInfo)){
                if(FileInfo.Attribute & FILE_ATTRIBUTE_DIRECTORY){
                    
                }else{
                    if(FsMakeDir((PBYTE)tmp_dst_path)){
                        DBGLINE(DBG_LVL, "MakeDir Success : tmp_dst_path=%s\n", tmp_dst_path);
                    }else{
                        return ERROR_CANNOT_MAKE_DIR;
                    }
                }
            }else{
                if(FsMakeDir((PBYTE)tmp_dst_path)){
                    DBGLINE(DBG_LVL, "MakeDir Success : tmp_dst_path=%s\n", tmp_dst_path);
                }else{
                    return ERROR_CANNOT_MAKE_DIR;
                }
            }

            Ret = CopyAllWithDir2(tmp_src_path, tmp_dst_path, hWnd, pbStop);
            if(Ret != ERROR_NONE){
                return Ret;
            }
            
        }else{
            char tmp_dst_file[MAX_PATH];
            char tmp_src_file[MAX_PATH];
           memset(tmp_dst_file, 0, MAX_PATH);
            memset(tmp_src_file, 0, MAX_PATH);

            strcpy(tmp_dst_file, dst_dir);
            
            strcpy(tmp_src_file, src_dir);
            
            
            DBGLINE(DBG_LVL, "File: FileData.Name=%s\n", FileData.Name);
            if(strlen(dst_dir)>0){                
                if(dst_dir[strlen(dst_dir)-1] != '/'){
                    strcat(tmp_dst_file, "/");
                }
                strcat(tmp_dst_file, (char*)FileData.Name);
            }else{
                return ERROR_UNKNOWN;
            }

            if(strlen(src_dir)>0){                
                if(src_dir[strlen(src_dir)-1] != '/'){
                    strcat(tmp_src_file, "/");
                }
                strcat(tmp_src_file, (char*)FileData.Name);
            }else{
                return ERROR_UNKNOWN;
            }
            
            DBGLINE(DBG_LVL, "File: tmp_dst_file=%s\n", tmp_dst_file);
            DBGLINE(DBG_LVL, "File: tmp_src_file=%s\n", tmp_src_file);

            if(FsGetFileInfo((PBYTE)tmp_dst_file, &FileInfo)){
                if(FileInfo.Attribute & FILE_ATTRIBUTE_DIRECTORY){
                    
                }else{
                    /*
                    char old_file_name[MAX_PATH];
                    memset(old_file_name, 0, MAX_PATH);
                    strcpy(old_file_name, tmp_dst_path);
                    strcat(old_file_name, ".old");
                    if(FsRename((PBYTE)old_file_name, (PBYTE)tmp_dst_path)){
                        
                    }else{
                        return ERROR_CANNOT_CHANGE_FILENAME;
                    }
                    */
                }
            }
            
            hSrcFile = FsOpen((PBYTE)tmp_src_file);
            if(hSrcFile == NULL){
                return ERROR_CANNOT_OPEN_FILE;
            }

#ifndef _USE_TEMP_FILE_
            hDstFile = FsOpen((PBYTE)tmp_dst_file);
            DBGLINE(DBG_LVL, "hDstFile=%x\n", hDstFile);
            if(hDstFile == NULL){
                BOOL    bRet;
                bRet = FsCreate((PBYTE)tmp_dst_file);
                DBGLINE(DBG_LVL, "hDstFile=%x\n", bRet);
                if(bRet == NULL){
                    return ERROR_CANNOT_CREATE_FILE;
                }
                hDstFile = FsOpen((PBYTE)tmp_dst_file);
                DBGLINE(DBG_LVL, "hDstFile=%x\n", hDstFile);
                if(hDstFile == NULL){
                    return ERROR_CANNOT_OPEN_FILE;
                }
            }

            do
            {
                INT32	nRead;

                nRead = FsRead( hSrcFile, TempBuf, MAX_TEMPBUF );
        //DEBUGMSG(0,"%s : FsRead : nRead = %d\n",__FUNCTION__, nRead );
                if( nRead <= 0 )    // end
                    break;

                do
                {
                    nRead -= FsWrite( hDstFile, TempBuf, nRead );
        //DEBUGMSG(0,"%s : FsWrite : nRead = %d\n",__FUNCTION__, nRead );
                }while(nRead > 0 );
            }while(1);

            
            if( hSrcFile ){
                
                FsClose( hSrcFile );
            }
                
            if( hDstFile){
                FsTruncate( hDstFile );
                FsClose( hDstFile );
            }

            iCopyFileCount++;
#if 1
            if(hWnd){
                //PostMessage(hWnd, WM_USER, WM_COPYFILECOUNT, iCopyFileCount);    
                SendMessage(hWnd, WM_USER, WM_COPYFILECOUNT, iCopyFileCount);    
            }
#endif
            
            DBGLINE(DBG_LVL, "iCopyFileCount=%d\n", iCopyFileCount);

#else
            memset(tmp_dst_file_name, 0, MAX_PATH);
            strcpy(tmp_dst_file_name, tmp_dst_file);
            strcat(tmp_dst_file_name, ".tmpcopyfile");
            DBGLINE(DBG_LVL, "tmp_dst_file_name: %s\n", tmp_dst_file_name);
            hDstFile = FsOpen((PBYTE)tmp_dst_file_name);
            DBGLINE(DBG_LVL, "hDstFile=%x\n", hDstFile);
            if(hDstFile == NULL){
                BOOL    bRet;
                bRet = FsCreate((PBYTE)tmp_dst_file_name);
                DBGLINE(DBG_LVL, "hDstFile=%x\n", bRet);
                if(bRet == NULL){
                    return ERROR_CANNOT_CREATE_FILE;
                }
                hDstFile = FsOpen((PBYTE)tmp_dst_file_name);
                DBGLINE(DBG_LVL, "hDstFile=%x\n", hDstFile);
                if(hDstFile == NULL){
                    return ERROR_CANNOT_OPEN_FILE;
                }
            }

            do
            {
                INT32	nRead;

                nRead = FsRead( hSrcFile, TempBuf, MAX_TEMPBUF );
        //DEBUGMSG(0,"%s : FsRead : nRead = %d\n",__FUNCTION__, nRead );
                if( nRead <= 0 )    // end
                    break;

                do
                {
                    nRead -= FsWrite( hDstFile, TempBuf, nRead );
        //DEBUGMSG(0,"%s : FsWrite : nRead = %d\n",__FUNCTION__, nRead );
                }while(nRead > 0 );
            }while(1);

            
            if( hSrcFile ){
                
                FsClose( hSrcFile );
            }
                
            if( hDstFile){
                FsTruncate( hDstFile );
                FsClose( hDstFile );
            }

            iCopyFileCount++;
#if 1
            if(hWnd){
                //PostMessage(hWnd, WM_USER, WM_COPYFILECOUNT, iCopyFileCount);    
                SendMessage(hWnd, WM_USER, WM_COPYFILECOUNT, iCopyFileCount);    
            }
#endif
            
            DBGLINE(DBG_LVL, "iCopyFileCount=%d\n", iCopyFileCount);
            INT32 ret;
            ret = FsDelete((PBYTE)tmp_dst_file);
            DBGLINE(DBG_LVL, "ret=%d\n", ret);
            
            DBGLINE(DBG_LVL, "tmp_dst_file=%s\n",tmp_dst_file);
            DBGLINE(DBG_LVL, "tmp_dst_file_name=%s\n",tmp_dst_file_name);
            
            ret = FsRename((PBYTE)tmp_dst_file, (PBYTE)tmp_dst_file_name);
            DBGLINE(DBG_LVL, "ret=%d\n", ret);
#endif
             FsSync();
            //ret = FsDelete((PBYTE)DstTempFileName);
            //DBGLINE(DBG_LVL, "ret=%d\n", ret);
            DBGLINE(DBG_LVL, "Fin!!!\n");
        }
    }
    
    return ERROR_NONE;
    
}


INT32 CopyAllWithDir( char* src_dir,  char* dst_dir)
{
    DBGLINE(DBG_LVL, "src_dir=%s\n", src_dir);
    DBGLINE(DBG_LVL, "dst_dir=%s\n", dst_dir);
    
    HANDLE hSrcFile;
    HANDLE hDstFile;

    INT32 Ret;
            
    char tmp_dst_path[MAX_PATH];
    char tmp_src_path[MAX_PATH];
    memset(tmp_dst_path, 0, MAX_PATH);
    memset(tmp_src_path, 0, MAX_PATH);
    strcpy(tmp_dst_path, dst_dir);
    strcpy(tmp_src_path, src_dir);
    
    
    FILEDATA FileData;
    FILEINFO FileInfo;
    
    int nItem;
    for( nItem = 0; FsReadDir( (PBYTE)src_dir, &FileData, nItem) != 0 ; nItem++ ){
        DBGLINE(DBG_LVL, "FileData.Name=%s\n", FileData.Name);
        
        
        if(strcmp((char*)FileData.Name, ".") == 0){
            continue;
        }else if(strcmp((char*)FileData.Name, "..") == 0){
            continue;
        }
        
        if(FileData.Attribute & FILE_ATTRIBUTE_DIRECTORY){
            DBGLINE(DBG_LVL, "Dir: FileData.Name=%s\n", FileData.Name);
            memset(tmp_src_path, 0, MAX_PATH);
            strcpy(tmp_src_path, src_dir);

            memset(tmp_dst_path, 0, MAX_PATH);
            strcpy(tmp_dst_path, dst_dir);
            
            if(strlen(dst_dir)>0){                
                if(dst_dir[strlen(dst_dir)-1] != '/'){
                    strcat(tmp_dst_path, "/");
                }
                strcat(tmp_dst_path, (char*)FileData.Name);
            }else{
                return ERROR_UNKNOWN;
            }

            if(strlen(src_dir)>0){                
                if(src_dir[strlen(src_dir)-1] != '/'){
                    strcat(tmp_src_path, "/");
                }
                strcat(tmp_src_path, (char*)FileData.Name);
            }else{
                return ERROR_UNKNOWN;
            }

            
            DBGLINE(DBG_LVL, "Dir: tmp_dst_path=%s\n", tmp_dst_path);
            DBGLINE(DBG_LVL, "Dir: tmp_src_path=%s\n", tmp_src_path);
            
            if(FsGetFileInfo((PBYTE)tmp_dst_path, &FileInfo)){
                if(FileInfo.Attribute & FILE_ATTRIBUTE_DIRECTORY){
                    
                }else{
                    if(FsMakeDir((PBYTE)tmp_dst_path)){
                        
                    }else{
                        return ERROR_CANNOT_MAKE_DIR;
                    }
                }
            }else{
                if(FsMakeDir((PBYTE)tmp_dst_path)){
                    
                }else{
                    return ERROR_CANNOT_MAKE_DIR;
                }
            }

            Ret = CopyAllWithDir(tmp_src_path, tmp_dst_path);
            if(Ret != ERROR_NONE){
                return Ret;
            }
            
        }else{
            char tmp_dst_file[MAX_PATH];
            char tmp_src_file[MAX_PATH];
            memset(tmp_dst_file, 0, MAX_PATH);
            memset(tmp_src_file, 0, MAX_PATH);

            strcpy(tmp_dst_file, dst_dir);
            
            strcpy(tmp_src_file, src_dir);
            
            
            DBGLINE(DBG_LVL, "File: FileData.Name=%s\n", FileData.Name);
            if(strlen(dst_dir)>0){                
                if(dst_dir[strlen(dst_dir)-1] != '/'){
                    strcat(tmp_dst_file, "/");
                }
                strcat(tmp_dst_file, (char*)FileData.Name);
            }else{
                return ERROR_UNKNOWN;
            }

            if(strlen(src_dir)>0){                
                if(src_dir[strlen(src_dir)-1] != '/'){
                    strcat(tmp_src_file, "/");
                }
                strcat(tmp_src_file, (char*)FileData.Name);
            }else{
                return ERROR_UNKNOWN;
            }
            
            DBGLINE(DBG_LVL, "File: tmp_dst_file=%s\n", tmp_dst_file);
            DBGLINE(DBG_LVL, "File: tmp_src_file=%s\n", tmp_src_file);

            if(FsGetFileInfo((PBYTE)tmp_dst_file, &FileInfo)){
                if(FileInfo.Attribute & FILE_ATTRIBUTE_DIRECTORY){
                    
                }else{
                    /*
                    char old_file_name[MAX_PATH];
                    memset(old_file_name, 0, MAX_PATH);
                    strcpy(old_file_name, tmp_dst_path);
                    strcat(old_file_name, ".old");
                    if(FsRename((PBYTE)old_file_name, (PBYTE)tmp_dst_path)){
                        
                    }else{
                        return ERROR_CANNOT_CHANGE_FILENAME;
                    }
                    */
                }
            }
            
            hSrcFile = FsOpen((PBYTE)tmp_src_file);
            if(hSrcFile == NULL){
                return ERROR_CANNOT_OPEN_FILE;
            }

#ifndef _USE_TEMP_FILE_

            hDstFile = FsOpen((PBYTE)tmp_dst_file);
            DBGLINE(DBG_LVL, "hDstFile=%x\n", hDstFile);
            if(hDstFile == NULL){
                BOOL bRet;
                bRet = FsCreate((PBYTE)tmp_dst_file);
                DBGLINE(DBG_LVL, "bRet=%x\n", bRet);
                if(bRet == FALSE){
                    return ERROR_CANNOT_CREATE_FILE;
                }

                hDstFile = FsOpen((PBYTE)tmp_dst_file);
                DBGLINE(DBG_LVL, "hDstFile=%x\n", hDstFile);
                if(hDstFile == NULL){
                    return ERROR_CANNOT_OPEN_FILE;
                }
            }

            do
            {
                INT32	nRead;

                nRead = FsRead( hSrcFile, TempBuf, MAX_TEMPBUF );
        //DEBUGMSG(0,"%s : FsRead : nRead = %d\n",__FUNCTION__, nRead );
                if( nRead <= 0 )    // end
                    break;

                do
                {
                    nRead -= FsWrite( hDstFile, TempBuf, nRead );
        //DEBUGMSG(0,"%s : FsWrite : nRead = %d\n",__FUNCTION__, nRead );
                }while(nRead > 0 );
            }while(1);

            
            if( hSrcFile ){
                
                FsClose( hSrcFile );
            }
                
            if( hDstFile){
                FsTruncate( hDstFile );
                FsClose( hDstFile );
            }

            iCopyFileCount++;
#if 0
            if(hWnd){
                PostMessage(hWnd, WM_USER, WM_COPYFILECOUNT, iCopyFileCount);    
            }
#endif
            
            
            DBGLINE(DBG_LVL, "iCopyFileCount=%d\n", iCopyFileCount);

#else

            memset(tmp_dst_file_name, 0, MAX_PATH);
            strcpy(tmp_dst_file_name, tmp_dst_file);
            strcat(tmp_dst_file_name, ".tmpcopyfile");
            DBGLINE(DBG_LVL, "tmp_dst_file_name: %s\n", tmp_dst_file_name);

            hDstFile = FsOpen((PBYTE)tmp_dst_file_name);
            DBGLINE(DBG_LVL, "hDstFile=%x\n", hDstFile);
            if(hDstFile == NULL){
                BOOL bRet;
                bRet = FsCreate((PBYTE)tmp_dst_file_name);
                DBGLINE(DBG_LVL, "bRet=%x\n", bRet);
                if(bRet == FALSE){
                    return ERROR_CANNOT_CREATE_FILE;
                }

                hDstFile = FsOpen((PBYTE)tmp_dst_file_name);
                DBGLINE(DBG_LVL, "hDstFile=%x\n", hDstFile);
                if(hDstFile == NULL){
                    return ERROR_CANNOT_OPEN_FILE;
                }
            }

            do
            {
                INT32	nRead;

                nRead = FsRead( hSrcFile, TempBuf, MAX_TEMPBUF );
        //DEBUGMSG(0,"%s : FsRead : nRead = %d\n",__FUNCTION__, nRead );
                if( nRead <= 0 )    // end
                    break;

                do
                {
                    nRead -= FsWrite( hDstFile, TempBuf, nRead );
        //DEBUGMSG(0,"%s : FsWrite : nRead = %d\n",__FUNCTION__, nRead );
                }while(nRead > 0 );
            }while(1);

            
            if( hSrcFile ){
                
                FsClose( hSrcFile );
            }
                
            if( hDstFile){
                FsTruncate( hDstFile );
                FsClose( hDstFile );
            }

            iCopyFileCount++;
#if 0
            if(hWnd){
                PostMessage(hWnd, WM_USER, WM_COPYFILECOUNT, iCopyFileCount);    
            }
#endif
            
            
            DBGLINE(DBG_LVL, "iCopyFileCount=%d\n", iCopyFileCount);
            
            
            INT32 ret;
            ret = FsRename((PBYTE)tmp_dst_file, (PBYTE)tmp_dst_file_name);
            DBGLINE(DBG_LVL, "ret=%d\n", ret);
            //ret = FsDelete((PBYTE)DstTempFileName);
            //DBGLINE(DBG_LVL, "ret=%d\n", ret);
#endif

            FsSync();
            DBGLINE(DBG_LVL, "Fin!!!\n");
        }
    }
    
    return ERROR_NONE;
    
}


static    char tmp_src_path[MAX_PATH];
VOID DeleteAllFiles(char* target_dir)
{
 

    FILEDATA FileData;
    int nItem;
            
    for( nItem = 0; FsReadDir( (PBYTE)target_dir, &FileData, nItem) != 0 ; nItem++ ){
        
        DBGLINE(DBG_LVL, "nItem=%d : FileName=%s\n",nItem, FileData.Name);
        if(strcmp((char*)FileData.Name, ".") == 0){
            
        }else if(strcmp((char*)FileData.Name, "..") == 0){
            
        }else{
            
            memset(tmp_src_path, 0, MAX_PATH);

            strcat(tmp_src_path, target_dir);
            strcat(tmp_src_path, "/");
            strcat(tmp_src_path, (char*)FileData.Name);

            DBGLINE(DBG_LVL, "tmp_src_path=%s\n", tmp_src_path);

            if(FsDelete((PBYTE)tmp_src_path)){
                nItem--;
                DBGLINE(DBG_LVL, "FsDelete success!!!\n");
            }else{
                DBGLINE(DBG_LVL, "FsDelete fail!!!\n");
            }
        }
    }
}


#if defined(__DOOROOS_DLL__)
#include <moduleloader.h>

PUBLIC BOOL
DllMain( HANDLE hDll, UINT32 Reason, PVOID Reserved )
{
//	STATIC PVOID pMalloc = NULL;
    switch( Reason )
    {
    case DLL_PROCESS_ATTACH:
    	break;
    
    case DLL_PROCESS_DETACH:
    	break;
    }

    return TRUE;
}
#endif
