/*   DOOROOS: util_zipugd.c   */

/*
 * Copyright (C) 2015    iBOSN systems
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted completely.
 *
 */

/*
 * Module Name: util_zipugd.c
 *
 * Purpose: Create the user application.
 *
 */

#include <types.h>
#include <thread.h>
#include <window.h>
#include <time.h>
#include <memory.h>
#include <fs.h>
#include <lib/stdio.h>
#include <lib/string.h>
#include <panic.h>

//#include <sound.h>
#include <dev/kbd.h>

#include <gl/sdi.h>
#include <moduleloader.h>
#include <resource.h>


// debug level info
#define DBG_LVL_UPGRADE     0
extern BOOL WidgetSetPos( PWIDGET pWidget, UINT32 Id, UINT32 xPos, UINT32 yPos, UINT32 Width, UINT32 Height);
extern VOID FontSet( PGL pGL, UINT32 FontSize );


//// file path info
//#define USB_MB_ZIPFILE_PATH 	"/nand/wav/"   // source folder
//#define MB_COPY_PATH  			"/Ram/"        // target position
//#define USB_ROOT_PATH 			"/Ram/"

#define MB_UPGRADE 1



#ifndef WM_COPYFILECOUNT
#define WM_COPYFILECOUNT                      7777
#endif 

#define TIMER_ID_FILECOPY          (1<<1)

#define STC_WARNING_TEXT                    84
#define STC_REBOOT_TEXT                     85

#define PROGRBAR_FILECOPY                   IDC_PROGBAR_DEF(81)

//#define STC_PASSWORD                            1031


// position info
#define GUI_WIDTH_MAX			100.0
#define GUI_HEIGHT_MAX			100.0
#define GUI_TOP_PERCENT			60.0
#define GUI_BOTTOM_PERCENT		40.0

#define GUI_TITLE_X      10.0
#define GUI_TITLE_Y      0.0
#define GUI_TITLE_WIDTH  80.0
#define GUI_TITLE_HEIGHT 10.0

#define GUI_DOWNLOAD_PERCENT_X      10.0
#define GUI_DOWNLOAD_PERCENT_Y      60.0
#define GUI_DOWNLOAD_PERCENT_WIDTH  80.0
#define GUI_DOWNLOAD_PERCENT_HEIGHT 10.0

#define GUI_DOWNLOAD_PROGRESS_X      10.0
#define GUI_DOWNLOAD_PROGRESS_Y      70.0
#define GUI_DOWNLOAD_PROGRESS_WIDTH  80.0
#define GUI_DOWNLOAD_PROGRESS_HEIGHT 20.0

#define GUI_BUTTON_START_X      10.0
#define GUI_BUTTON_START_Y      20.0
#define GUI_BUTTON_START_WIDTH  30.0
#define GUI_BUTTON_START_HEIGHT 30.0

#define GUI_BUTTON_ABORT_X      60.0
#define GUI_BUTTON_ABORT_Y      20.0
#define GUI_BUTTON_ABORT_WIDTH  30.0
#define GUI_BUTTON_ABORT_HEIGHT 30.0

// color info
#define USBDL_COLOR_TOP 				LT_GRAY
#define USBDL_COLOR_BOTTOM				LT_BLUE
#define USBDL_COLOR_BUTTON				LT_GRAY
#define USBDL_COLOR_TITLE_TEXT 			BLACK
#define USBDL_COLOR_PROGRESS_TEXT 		WHITE
#define USBDL_COLOR_PROGRESSBAR_BACK  	LT_GRAY
#define USBDL_COLOR_PROGRESSBAR 		GREEN



STATIC struct __FILECOPY_SETTING__
{
	INT32 option;
    //
}MbUpgradeSetting;

static HWND hWndZipUpgrade;


//STATIC int downloadPercent = 0; // 0 ~ 100%
STATIC BOOL stopDownload = FALSE;
STATIC int totalUpgradeFiles = 0;
STATIC int currentUpgradedFiles = 0;

STATIC UINT32 DialogMbUpgradeWndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );

STATIC VOID Widget_DrawDialogMbUpgradeBg( PGL pGL, WIDGET_PDRAW_BACKGROUND pDrawBackground );
STATIC VOID Widget_DrawDownloadPercentText( PGL pGL, WIDGET_PDRAW_STATIC pDrawImage);
STATIC VOID Widget_DrawZip_Button( PGL pGL, WIDGET_PDRAW_BUTTON pDrawButton );
STATIC VOID Widget_DrawProgressBarZipPercent( PGL pGL, WIDGET_PDRAW_PROGRESSBAR pDrawProgressBar);

INT32 (*m_Unzipfile)(const char* src_file_path, const char* dst_dir_path, HWND hWnd, int *pTotal, BOOL *pStop );


STATIC INT32 th_Install(UINT Arg);
//======================================================
STATIC HANDLE hFileCopyThread = NULL;
#define STACK_SIZE      (4096*16)
STATIC THREADCONFIG     FileCopyThread = 
                                    {
                                        NULL, 
                                        th_Install, 
                                        (PBYTE)NULL, 
                                        STACK_SIZE, 
                                        THREAD_PRIORITY_USER_HIGH + 3
                                    };
//======================================================

//#define SETTING_TEXT_FONT_SIZE 30
STATIC BOOL bDisktAccess = FALSE;

STATIC GL               Gl;
//PUBLIC PSDI             pSDIVirDialogLeft = NULL;

//#define BTN_PASSWORD_WIDTH              67
//#define BTN_PASSWORD_HEIGHT             67
//#define BTN_PASSWORD_START_X            50+50
//#define BTN_PASSWORD_START_X2           125+50
//#define BTN_PASSWORD_START_X3           200+50
//#define BTN_PASSWORD_START_X4           290+50
//#define BTN_PASSWORD_START_Y            150
//#define BTN_PASSWORD_START_Y2           225
//#define BTN_PASSWORD_START_Y3           300
//#define BTN_PASSWORD_START_Y4           375
//
//#define BTN_OK_WDITH           134
//#define BTN_OK_HEIGHT          67

// dialog proc info
STATIC WNDCLASS         DialogZipWndClass = { CS_DBLCLKS | CS_VREDRAW | CS_HREDRAW, (PWNDPROC)DialogMbUpgradeWndProc };

// gui position info
STATIC WIDGET           DialogZipWndWidget[] =
{	
	// background
	{WIDGET_TYPE_BACKGROUND, 
        IDC_BK_RSVD, 
                WIDGET_RECT(  0, 0, GUI_WIDTH_MAX, GUI_HEIGHT_MAX), 
                NULL, 
		        (PWIDDRAWPROC)Widget_DrawDialogMbUpgradeBg, &Gl, 
                {{NULL, NULL}}},
	
	// start button
	{WIDGET_TYPE_PUSHBUTTON, 
		IDC_BUTTON_DEF(1),
			WIDGET_RECT( GUI_BUTTON_START_X,  GUI_BUTTON_START_Y,  GUI_BUTTON_START_WIDTH,  GUI_BUTTON_START_HEIGHT), NULL, 
			(PWIDDRAWPROC)Widget_DrawZip_Button, &Gl, 
			{{"Start", NULL, NULL }}},
					
    // abort button
	{WIDGET_TYPE_PUSHBUTTON, 
		IDC_BUTTON_DEF(2),
			WIDGET_RECT( GUI_BUTTON_ABORT_X,  GUI_BUTTON_ABORT_Y,  GUI_BUTTON_ABORT_WIDTH,  GUI_BUTTON_ABORT_HEIGHT), NULL, 
			(PWIDDRAWPROC)Widget_DrawZip_Button, &Gl, 
			{{"Abort", NULL, NULL }}},


    // download percent text
	{WIDGET_TYPE_STATIC, 
        IDC_STATIC_DEF(4), 
            WIDGET_RECT(  GUI_DOWNLOAD_PERCENT_X,  GUI_DOWNLOAD_PERCENT_Y, GUI_DOWNLOAD_PERCENT_WIDTH, GUI_DOWNLOAD_PERCENT_HEIGHT), 
            NULL, 
            (PWIDDRAWPROC)Widget_DrawDownloadPercentText, &Gl, 
            {{"0 %%", NULL,}}},
			
	// progressbar
	{WIDGET_TYPE_PROGRESSBAR, 
        PROGRBAR_FILECOPY, 
			WIDGET_RECT( GUI_DOWNLOAD_PROGRESS_X, GUI_DOWNLOAD_PROGRESS_Y, GUI_DOWNLOAD_PROGRESS_WIDTH, GUI_DOWNLOAD_PROGRESS_HEIGHT), NULL,  
            (PWIDDRAWPROC)Widget_DrawProgressBarZipPercent, &Gl, 
            {{NULL, NULL,NULL }}},		

//#if 0
//    {WIDGET_TYPE_PROGRESSBAR, 
//            PROGRBAR_FILECOPY, 
//                WIDGET_RECT( 50, 120, 500, 80), NULL,  
//                (PWIDDRAWPROC)Widget_DrawProgressBarPrivate, &Gl, 
//                 {{NULL, NULL,NULL }}},
//
//    {WIDGET_TYPE_STATIC, 
//        IDC_STATIC_DEF(STC_WARNING_TEXT), 
//                        WIDGET_RECT(  50,  200, 500,50), 
//                        NULL, 
//                        (PWIDDRAWPROC)Widget_DrawWarningText, &Gl, 
//                        {{"CAN NOT ACCESS USB", NULL,}}},
//
//    {WIDGET_TYPE_STATIC, 
//        IDC_STATIC_DEF(STC_REBOOT_TEXT), 
//                        WIDGET_RECT(  50,  200, 500,50), 
//                        NULL, 
//                        (PWIDDRAWPROC)Widget_DrawRebootText, &Gl, 
//                        {{"Please, Rebooting System !", NULL,}}},
//#endif

    {-1,}
};

//=======================================================================================



// iProgressbarPose means percent(0 to 100%)
STATIC VOID ProgressBarUpdate(HWND hWnd, INT32 iProgressbarPose)
{    
    DBGLINE(DBG_LVL_UPGRADE, "hWnd=%x\n", hWnd);
    DBGLINE(DBG_LVL_UPGRADE, "iProgressbarPose=%d\n", iProgressbarPose);

        SendWidgetMessage(hWnd, PROGRBAR_FILECOPY, PBM_SETRANGE32, 0, totalUpgradeFiles);
        //SendWidgetMessage(hWndZipUpgrade, PROGRBAR_FILECOPY, PBM_SETPOS, 0, 0);
        
        SendWidgetMessage(hWnd, PROGRBAR_FILECOPY, PBM_SETPOS, iProgressbarPose, 0);
        INT32 iGetPose;
        
        iGetPose = SendWidgetMessage(hWnd, PROGRBAR_FILECOPY, PBM_GETPOS, 0, 0);
        DBGLINE(DBG_LVL_UPGRADE, "iGetPose=%d\n", iGetPose);
        //UpdateWidget(hWnd, IDC_PROGBAR_DEF(PROGRBAR_FILECOPY));
        
        PBRANGE range;
        
        SendWidgetMessage( hWnd, PROGRBAR_FILECOPY, PBM_GETRANGE, NULL,  (LPARAM)&range  );
        DBGLINE(DBG_LVL_UPGRADE, "range.iLow=%d\n", range.iLow);
        DBGLINE(DBG_LVL_UPGRADE, "range.iHigh=%d\n", range.iHigh);
        if(range.iHigh <= iProgressbarPose)
        {
//            //ShowWindow(hWnd, SW_HIDE);
//            ShowWidget(hWnd, PROGRBAR_FILECOPY, SW_HIDE);
//            SendWidgetMessage(hWnd, PROGRBAR_FILECOPY, PBM_SETPOS, 0, 0);
//           //ShowWidget(hWnd, IDC_STATIC_DEF(STC_REBOOT_TEXT), SW_SHOW);            
        }
}


STATIC INT32 CountUpgradeFiles(char* src_dir)
{
    DBGLINE(DBG_LVL_UPGRADE, "src_dir=%s\n", src_dir);
    

    EXTERN BOOL     FsGetFileInfo( PBYTE pPathName, PFILEINFO pFileInfo ); 
    

    FILEDATA FileData;

    char tmp_path[MAX_PATH];
    
    memset(tmp_path, 0, MAX_PATH);
    strcpy(tmp_path, src_dir);
    
    int iFileCount = 0;
DBGLINE(DBG_LVL_UPGRADE, "iFileCount=%d\n", iFileCount);

    int nItem;
    for( nItem = 0; FsReadDir( (PBYTE)src_dir, &FileData, nItem) != 0 ; nItem++ ){
        DBGLINE(DBG_LVL_UPGRADE, "FileInfo.Name=%s\n", FileData.Name);
        if(strcmp((char*)FileData.Name, ".") == 0){
            continue;
        }else if(strcmp((char*)FileData.Name, "..") == 0){
            continue;
        }
        
        if(FileData.Attribute & FILE_ATTRIBUTE_DIRECTORY){
            memset(tmp_path, 0, MAX_PATH);
            strcpy(tmp_path, src_dir);
            strcat(tmp_path, "/");
            strcat(tmp_path, (char*)FileData.Name);
            
            iFileCount += CountUpgradeFiles(tmp_path);
            DBGLINE(DBG_LVL_UPGRADE, "iFileCount=%d\n", iFileCount);
        }else{

            iFileCount++;
            DBGLINE(DBG_LVL_UPGRADE, "iFileCount=%d\n", iFileCount);
        }
    }
	
    DBGLINE(DBG_LVL_UPGRADE, "iFileCount=%d\n", iFileCount);
    return iFileCount;
}

STATIC char    SrcPath[MAX_PATH];
STATIC char    DstPath[MAX_PATH];

#define USB_MB_ZIPFILE_PATH 	"/usbdisk4/dooroo_fw/upgrade.zip"   // source zipfile path
#define MB_COPY_PATH  			"/nand/"            // target position
//#define USB_ROOT_PATH 			"/Ram/"


STATIC INT32 th_Install(UINT Arg)
{
	DBGLINE(DBG_LVL_UPGRADE, "th_Install start \n");
    INT32 Ret = 0;

    strcpy( SrcPath, USB_MB_ZIPFILE_PATH);
    strcpy( DstPath, MB_COPY_PATH);

    UINT32 bRet;
    FILEINFO    FileInfo;
    bRet = FsGetFileInfo( SrcPath, &FileInfo );
    if( !bRet )
        return;

    if( FileInfo.Attribute & FILE_ATTRIBUTE_DIRECTORY )
    	return;

	//Fs_emptyDir(DstPath);
	m_Unzipfile(SrcPath, DstPath, hWndZipUpgrade, &totalUpgradeFiles, &stopDownload);


    
    if(hFileCopyThread){
        ThreadTerminate(hFileCopyThread);
        hFileCopyThread = NULL;    
    }    

	DBGLINE(DBG_LVL_UPGRADE, "th_Install ret(%d) end \n", Ret);
    return Ret;
}

STATIC BOOL StartUpgradeZipFile( HWND hWnd )
{
#if 1
//    DBGLINE(DBG_LVL_UPGRADE, "hWnd=%x\n", hWnd);
////    BYTE Buf[512];

	stopDownload = FALSE;

//    /* copy the upgrade file into the internal filesystem */
//    if(0 != m_CheckDstDir(USB_ROOT_PATH)){
//        DBGLINE(DBG_LVL_UPGRADE, "USB_ROOT_PATH cannot access !!!\n");
//        return FALSE;
//    }
//
//    if(0 != m_CheckDstDir(USB_MB_ZIPFILE_PATH)){
//        DBGLINE(DBG_LVL_UPGRADE, "USB_MB_ZIPFILE_PATH cannot access !!!\n");
//        return FALSE;
//    }
//
//
//    
//DBGLINE(DBG_LVL_UPGRADE, "totalUpgradeFiles=%d\n", totalUpgradeFiles);
//    totalUpgradeFiles = CountUpgradeFiles(USB_MB_ZIPFILE_PATH);
//DBGLINE(DBG_LVL_UPGRADE, "totalUpgradeFiles=%d\n", totalUpgradeFiles);
//    if(totalUpgradeFiles <= 0){
//        return FALSE;
//    }
//
//    m_MakeDstDir(MB_COPY_PATH);
	hFileCopyThread = ThreadCreate(&FileCopyThread);
    //UpdateWidget(hWnd, PROGRBAR_FILECOPY);    

    //hFileCopyThread = ThreadCreate(&FileCopyThread);
    //DBGLINE(DBG_LVL_UPGRADE, "hFileCopyThread=%x\n", hFileCopyThread);
#else
	
	stopDownload = FALSE;
//	downloadPercent = 0;
	currentUpgradedFiles = 0;				
	totalUpgradeFiles = CountUpgradeFiles(USB_MB_ZIPFILE_PATH);
				
	UpdateWidget(hWnd, IDC_STATIC_DEF(4));  
	UpdateWidget(hWnd, PROGRBAR_FILECOPY);  
	
	m_MakeDstDir(MB_COPY_PATH);
	hFileCopyThread = ThreadCreate(&FileCopyThread);
#endif 
    return TRUE;
}

STATIC BOOL StopUpgradeZipFile( HWND hWnd )
{
	stopDownload = TRUE;
	UpdateWidget(hWnd, IDC_STATIC_DEF(4));  
	UpdateWidget(hWnd, PROGRBAR_FILECOPY);  
}


STATIC HANDLE hTimer_MBUPGRADE = NULL;

STATIC UINT32 DialogMbUpgradeWndProc(
  HWND hWnd,      // handle to window
  UINT uMsg,      // message identifier
  WPARAM wParam,  // first message parameter
  LPARAM lParam   // second message parameter
)
{
    switch( uMsg )
    {
    case WM_USER:{
        if(wParam == WM_COPYFILECOUNT){
			currentUpgradedFiles =  lParam;
			DBGLINE(DBG_LVL_UPGRADE, "currentUpgradedFiles=%d \n",currentUpgradedFiles);
			DBGLINE(DBG_LVL_UPGRADE, "hWnd = %x, %d \n", hWnd, hWnd);
#if 1
            ProgressBarUpdate (hWnd, currentUpgradedFiles );
            UpdateWidget(hWnd, IDC_STATIC_DEF(4));

#else
			int downloadPercentTemp = (100 * currentUpgradedFiles)/totalUpgradeFiles;
			DBGLINE(DBG_LVL_UPGRADE, "downloadPercentTemp=%d \n",downloadPercent);
			BOOL bDiff = FALSE;
			if(downloadPercentTemp != downloadPercent)
				bDiff = TRUE;			
			downloadPercent = downloadPercentTemp;
			if(bDiff == TRUE)
			{
				UpdateWidget(hWnd, IDC_STATIC_DEF(4));  
				UpdateWidget(hWnd, PROGRBAR_FILECOPY);  
			}
#endif
        }
    }break;

    case WM_CREATE:
    {
		DBGLINE(DBG_LVL_UPGRADE, "hWnd = %x, %d \n", hWnd, hWnd);
        MbUpgradeSetting.option = -1;
        
        SendWidgetMessage(hWnd, PROGRBAR_FILECOPY, PBM_SETRANGE32, 0, 100);
        SendWidgetMessage(hWnd, PROGRBAR_FILECOPY, PBM_SETPOS, 0, 0);
		SendWidgetMessage(hWnd, PROGRBAR_FILECOPY, PBM_SETSTEP, 1, 0);		

        if( hTimer_MBUPGRADE == NULL)
		{
			DBGLINE(DBG_LVL_UPGRADE, "create timer \n");
            hTimer_MBUPGRADE = TimerCreate( hWnd, TIMER_ID_FILECOPY, 500, NULL );
        }
        
        
    }break;    

    case WM_TIMER:{
		//DBGLINE(DBG_LVL_UPGRADE, "WM_TIMER call \n");
        if( wParam & TIMER_ID_FILECOPY){
            
			//DBGLINE(DBG_LVL_UPGRADE, "TIMER_ID_FILECOPY call \n");
            BOOL bRet;
            
            if(hTimer_MBUPGRADE){
                TimerStop(hTimer_MBUPGRADE);
            }

            /* automatically start the upgrade process */
            if(MbUpgradeSetting.option == MB_UPGRADE)
            {
                bRet = StartUpgradeZipFile(hWnd);
                if(!bRet){
                    //ShowWidget(hWnd, IDC_STATIC_DEF(STATIC_WARNING_TEXT), SW_SHOW);
                    //UpdateWidget(hWnd, IDC_STATIC_DEF(STATIC_WARNING_TEXT));
                    //Sleep(200);
                    ShowWindow(hWnd, SW_HIDE);
                }
            }
        }
    }break;
    
    case WM_COMMAND:
    {
        switch( LOWORD(wParam) )
        {
            //case IDC_BUTTON_DEF(BUTTON_CLOSE):{
            //    switch(HIWORD(wParam))
            //    {
            //        case BN_CLICKED :{
            //                    
            //            PlaySound((PBYTE)"/nand/wav/keysound.wav", 0);
            //            ShowWindow(hWnd, SW_HIDE);
            //            
            //            
            //            
            //        }break;
            //    }//switch end
            //    
            //}break;  
            
            case IDC_BUTTON_DEF(1):
			{
                DBGLINE(DBG_LVL_UPGRADE, "Click Start!!! \n");  
				StartUpgradeZipFile(hWnd);				
            }break;
			
            case IDC_BUTTON_DEF(2):
			{	
				//downloadPercent = 0;
				//currentUpgradedFiles = 0;
				//totalUpgradeFiles = 0;				
				StopUpgradeZipFile(hWnd);
				DBGLINE(DBG_LVL_UPGRADE, "Click Abort!!! stopDownload(%x, %d) \n", &stopDownload, stopDownload);
            }break;

        }//switch end
    }break;
        
    case WM_VSCROLL:
    case WM_HSCROLL:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_HSCROLL wParam = %x\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_HSCROLL lParam = %x\n",__FUNCTION__, lParam );
        
		break;
         
    case WM_KEYDOWN:
    {
        HWND    hWndFocus;

        hWndFocus = GetFocusWindow(hWnd);

        if (hWndFocus != hWnd)
        {
            SendMessage( hWndFocus, uMsg, wParam, lParam );
            break;
        }

        break;
    }
    
    case WM_KEYUP:
    {
        HWND    hWndFocus;

        hWndFocus = GetFocusWindow(hWnd);
        if (hWndFocus != hWnd)
        {
            SendMessage( hWndFocus, uMsg, wParam, lParam );
            break;
        }

        break;
    }
    case WM_CHAR:
        break;
        
    case WM_MOUSEMOVE:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_MOUSEMOVE wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_MOUSEMOVE lParam = %X\n",__FUNCTION__, lParam );
    	break;

    case WM_LBUTTONDOWN:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_LBUTTONDOWN wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_LBUTTONDOWN lParam = %X\n",__FUNCTION__, lParam );
    	break;
    case WM_LBUTTONUP:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_LBUTTONUP wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_LBUTTONUP lParam = %X\n",__FUNCTION__, lParam );
    	break;
    case WM_LBUTTONDBLCLK:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_LBUTTONDCLK wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_LBUTTONDCLK lParam = %X\n",__FUNCTION__, lParam );
    	break;

    case WM_RBUTTONDOWN:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_RBUTTONDOWN wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_RBUTTONDOWN lParam = %X\n",__FUNCTION__, lParam );
    	break;
    case WM_RBUTTONUP:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_RBUTTONUP wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_RBUTTONUP lParam = %X\n",__FUNCTION__, lParam );
    	break;
    case WM_RBUTTONDBLCLK:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_RBUTTONDCLK wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_RBUTTONDCLK lParam = %X\n",__FUNCTION__, lParam );
    	break;

    case WM_MBUTTONDOWN:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_MBUTTONDOWN wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_MBUTTONDOWN lParam = %X\n",__FUNCTION__, lParam );
    	break;
    case WM_MBUTTONUP:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_MBUTTONUP wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_MBUTTONUP lParam = %X\n",__FUNCTION__, lParam );
    	break;
    case WM_MBUTTONDBLCLK:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_MBUTTONDBLCLK wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_MBUTTONDBLCLK lParam = %X\n",__FUNCTION__, lParam );
    	break;

    case WM_MOUSEWHEEL:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_MOUSEWHEEL wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_MOUSEWHEEL lParam = %X\n",__FUNCTION__, lParam );
    	break;

    case WM_SHOWWINDOW:
        DEBUGMSG(0,"%s  WM_SHOWWINDOW wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(0,"%s  WM_SHOWWINDOW lParam = %X\n",__FUNCTION__, lParam );
        if(wParam)
        { // SW_SHOW
            SetWindowCapture( hWnd ); 
            ShowWidget(hWnd, IDC_STATIC_DEF(STC_WARNING_TEXT), SW_HIDE);
            ShowWidget(hWnd, IDC_STATIC_DEF(STC_REBOOT_TEXT), SW_HIDE);
            if(hTimer_MBUPGRADE){
                TimerStart(hTimer_MBUPGRADE);
            }
        }
        else
        { // SW_HIDE
            ReleaseWindowCapture();
            ShowWidget(hWnd, IDC_STATIC_DEF(STC_WARNING_TEXT), SW_HIDE);
            ShowWidget(hWnd, IDC_STATIC_DEF(STC_REBOOT_TEXT), SW_HIDE);
            bDisktAccess = FALSE;
            MbUpgradeSetting.option = -1;
            if(hTimer_MBUPGRADE){
                TimerStop(hTimer_MBUPGRADE);
            }
        }
        
        break;

    case WM_ACTIVATE:
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_ACTIVATE wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_ACTIVATE lParam = %X\n",__FUNCTION__, lParam );
        break;

    case WM_SETFOCUS:
        DEBUGMSG(0,"%s  WM_SETFOCUS wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(0,"%s  WM_SETFOCUS lParam = %X\n",__FUNCTION__, lParam );
        
        
        break;

    case WM_KILLFOCUS:
        DEBUGMSG(0,"%s  WM_KILLFOCUS wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(0,"%s  WM_KILLFOCUS lParam = %X\n",__FUNCTION__, lParam );
        
        break;

    case WM_DESTROY:
        DEBUGMSG(DBG_LVL_UPGRADE,"A %s  WM_DESTROY wParam = %X\n",__FUNCTION__, wParam );
        DEBUGMSG(DBG_LVL_UPGRADE,"%s  WM_DESTROY lParam = %X\n",__FUNCTION__, lParam );
                
        break;

    default:
        break;
    }

    return 0;   
}

STATIC VOID ResizeWidgets_ZipFile(UINT32 Xres, UINT32 Yres)
{
    WidgetSetPos( DialogZipWndWidget, IDC_BK_RSVD, 0, 0, Xres, Yres);
	
//	WidgetSetPos( DialogZipWndWidget, IDC_STATIC_DEF(1), 0, 0, Xres,
//	Yres*(GUI_TOP_PERCENT/100.0));
//	
//	WidgetSetPos( DialogZipWndWidget, IDC_STATIC_DEF(2), 0, Yres*(GUI_TOP_PERCENT/100.0),
//	Xres, 
//	Yres*(GUI_BOTTOM_PERCENT/100.0));
//	
//	WidgetSetPos( DialogZipWndWidget, IDC_STATIC_DEF(3), 
//        Xres * (GUI_TITLE_X / 100.0), 
//        (GUI_TITLE_Y/100.0) * Yres,
//        Xres*(GUI_TITLE_WIDTH / GUI_WIDTH_MAX), 
//        Yres*(GUI_TITLE_HEIGHT / GUI_HEIGHT_MAX) );
	
	WidgetSetPos( DialogZipWndWidget, IDC_BUTTON_DEF(1), 
        Xres * (GUI_BUTTON_START_X / 100.0), 
        (GUI_BUTTON_START_Y/100.0) * Yres,
        Xres*(GUI_BUTTON_START_WIDTH / GUI_WIDTH_MAX), 
        Yres*(GUI_BUTTON_START_HEIGHT / GUI_HEIGHT_MAX) );
		
	WidgetSetPos( DialogZipWndWidget, IDC_BUTTON_DEF(2), 
        Xres * (GUI_BUTTON_ABORT_X / 100.0), 
        (GUI_BUTTON_ABORT_Y/100.0) * Yres,
        Xres*(GUI_BUTTON_ABORT_WIDTH / GUI_WIDTH_MAX), 
        Yres*(GUI_BUTTON_ABORT_HEIGHT / GUI_HEIGHT_MAX) );	
		
	WidgetSetPos( DialogZipWndWidget, IDC_STATIC_DEF(4), 
        Xres * (GUI_DOWNLOAD_PERCENT_X / 100.0), 
        (GUI_DOWNLOAD_PERCENT_Y/100.0) * Yres,
        Xres*(GUI_DOWNLOAD_PERCENT_WIDTH / GUI_WIDTH_MAX), 
        Yres*(GUI_DOWNLOAD_PERCENT_HEIGHT / GUI_HEIGHT_MAX) );	
		
	WidgetSetPos( DialogZipWndWidget, PROGRBAR_FILECOPY, 
        Xres * (GUI_DOWNLOAD_PROGRESS_X / 100.0), 
        (GUI_DOWNLOAD_PROGRESS_Y/100.0) * Yres,
        Xres*(GUI_DOWNLOAD_PROGRESS_WIDTH / GUI_WIDTH_MAX), 
        Yres*(GUI_DOWNLOAD_PROGRESS_HEIGHT / GUI_HEIGHT_MAX) );		
		
	DBGLINE(DBG_LVL_UPGRADE, "Button2(%d, %d, %d, %d) \n", 
	Xres * (GUI_BUTTON_START_X / 100.0),
	(GUI_BUTTON_START_Y/100.0) * Yres,
	Xres*(GUI_BUTTON_START_WIDTH / GUI_WIDTH_MAX),
	Yres*(GUI_BUTTON_START_HEIGHT / GUI_HEIGHT_MAX));
	
	//WidgetSetPos( DialogZipWndWidget, IDC_BUTTON_DEF(2), 
    //    Xres * (GUI_BUTTON_ABORT_WIDTH/100.0) * Xres, 
    //    (GUI_BUTTON_ABORT_Y/100.0) * Yres,
    //    Xres*(GUI_BUTTON_ABORT_WIDTH/GUI_WIDTH_MAX), 
    //    Yres*(GUI_BUTTON_ABORT_HEIGHT/GUI_HEIGHT_MAX));
	
	DBGLINE(DBG_LVL_UPGRADE, "Xres(%d), Yres(%d), po1(%d) \n", Xres, Yres, Yres*(GUI_TOP_PERCENT/100.0));
  
   // WidgetSetPos( DialogZipWndWidget, IDC_BUTTON_DEF(BUTTON_CLOSE), 
              //   Xres*(600-100)/800, Yres*(77-70)/600, Xres*100/800, Yres*80/600);
   
}


			
PUBLIC HWND CreateWindow_ZipWnd( HWND hParentWnd )
{
	DBGLINE(DBG_LVL_UPGRADE, "CreateWindow_DialogMbUpgradeWnd start@_@\n");

    HWND    hWnd;
    PSDI    pSDI;

	
	// load dllexport
	HANDLE hDll = LoadLibrary ((PBYTE)"util.dll");
    if( hDll == NULL)
	{
		DBGLINE(DBG_LVL_UPGRADE, "load util.dll fail. \n");
        return;
	}
	DBGLINE(DBG_LVL_UPGRADE, "load utility dll success. \n");
	
	m_Unzipfile = (PVOID)GetProcAddress( hDll, (PBYTE)"Unzipfile");

    GL_Init(&Gl);
    
    pSDI = GL_GetSDI(&Gl);

	DBGLINE(DBG_LVL_UPGRADE, "pSDI->Xres(%d) pSDI->Yres(%d) \n", pSDI->Xres, pSDI->Yres);
    ResizeWidgets_ZipFile(pSDI->Xres, pSDI->Yres);
	
	//hWnd = CreateWindow( WndOpen_WndWidget, 
	//				&WndOpen_WndClass, 
    //                  0|WS_VISIBLE | WS_COMPATDRAW, 
    //                  pSDI->Xres/4, 
	//				 pSDI->Yres/8, 
	//				 Width, Height,
    //                  NULL, NULL );
	
    hWndZipUpgrade = CreateWindow( DialogZipWndWidget, &DialogZipWndClass, 0/*|WS_VISIBLE*/, 0, 0, pSDI->Xres, pSDI->Yres, NULL, NULL );

	DBGLINE(DBG_LVL_UPGRADE, "CreateWindow_DialogMbUpgradeWnd end\n");
    
    return hWndZipUpgrade;
}



STATIC VOID
Widget_DrawDialogMbUpgradeBg( PGL pGL, WIDGET_PDRAW_BACKGROUND pDrawBackground )
//Widget_DrawImageBase( PGL pGL, WIDGET_PDRAW_STATIC pDrawImage)
{   
//DBGLINE(DBG_LVL_UPGRADE, "Widget_DrawDialogMbUpgradeBg start \n"); 
    RECT   Rect;    
    Rect = pDrawBackground->Rect;
    UINT32  Div;
    Div = Rect.Ymax * 0.6;

    Rect.Ymax = Div;
    GL_SetForeColor( pGL, USBDL_COLOR_TOP);
    GL_RectFill( pGL, &Rect );    

    Rect = pDrawBackground->Rect;
    Rect.Ymin = Div;
    GL_SetForeColor( pGL, USBDL_COLOR_BOTTOM);
    GL_RectFill( pGL, &Rect );    
	
//DBGLINE(DBG_LVL_UPGRADE, "Rect(%d, %d, %d, %d) \n", Rect.Xmin, Rect.Xmax,
	//Rect.Ymin, Rect.Ymax);
    
//    GL_SetForeColor( pGL, USBDL_COLOR_BACKGROUND);
//    GL_RectFill( pGL, &Rect );    
	
	//DBGLINE(DBG_LVL_UPGRADE, "Widget_DrawDialogMbUpgradeBg end \n");

    /* draw title text */
    GL_SetBkMode( pGL, GL_TRANSPARENCY_TRANSPARENT);//GL_TRANSPARENCY_TRANSPARENT , GL_TRANSPARENCY_OPAQUE
    GL_FontSetAlign( pGL, FONT_VCENTER|FONT_CENTER);
    
    GL_SetForeColor( pGL, USBDL_COLOR_TITLE_TEXT ); // set black

	// set font size
	PSDI    pSDI;
	pSDI = GL_GetSDI(pGL);

    // set position
    Rect.Xmin = pSDI->Xres * GUI_TITLE_X / 100;
    Rect.Ymin = pSDI->Xres * GUI_TITLE_Y / 100;
    Rect.Xmax = pSDI->Xres * GUI_TITLE_WIDTH / 100;
    Rect.Ymax = pSDI->Xres * GUI_TITLE_HEIGHT / 100;
    
	int nWeight = 6;
	int nValue = pSDI->Xres/100;
	//DBGLINE(DBG_LVL_UPGRADE, "Title font size(%d) \n",  nValue * nWeight );
    FontSet( pGL, nValue * nWeight );

    GL_TextDraw( pGL, &Rect, "Utility for the unzip" );    
 
    return;
}


STATIC VOID Widget_DrawDownloadPercentText( PGL pGL, WIDGET_PDRAW_STATIC pDrawImage)
{
	//DBGLINE(DBG_LVL_UPGRADE, "Widget_DrawDownloadPercentText start \n");
    RECT    Rect = pDrawImage->Rect;
	
	// draw background
	GL_SetForeColor( pGL, USBDL_COLOR_BOTTOM);
    GL_RectFill( pGL, &Rect );
                    
    GL_SetBkMode( pGL, GL_TRANSPARENCY_TRANSPARENT);//GL_TRANSPARENCY_TRANSPARENT , GL_TRANSPARENCY_OPAQUE
    //GL_FontSetAlign( pGL, FONT_VCENTER|FONT_LEFT);
    GL_FontSetAlign( pGL, FONT_VCENTER|FONT_CENTER);
    
    GL_SetForeColor( pGL, USBDL_COLOR_PROGRESS_TEXT ); // set black

	// set font size
	PSDI    pSDI;
	pSDI = GL_GetSDI(pGL);
		
	int nWeight = 5;
	int nValue = pSDI->Xres/100.0;
	// DBGLINE(DBG_LVL_UPGRADE, "%d\% \n",  downloadPercent );
    FontSet( pGL, nValue * nWeight );
	char szDownloadPercent[20] = "";
	sprintf(&szDownloadPercent, "%d / %d", currentUpgradedFiles, totalUpgradeFiles);
    GL_TextDraw( pGL, &Rect, &szDownloadPercent);    
}

STATIC VOID
Rect3DDraw(PGL pGL, PRECT pRect )
{
    UINT32      cColor;

    GL_LineDraw( pGL, pRect->Xmin, pRect->Ymin, pRect->Xmax, pRect->Ymin);
    GL_LineDraw( pGL, pRect->Xmin, pRect->Ymin, pRect->Xmin, pRect->Ymax);

    cColor = pGL->cFore;
    pGL->cFore = pGL->cBack;
    GL_LineDraw( pGL, pRect->Xmin, pRect->Ymax, pRect->Xmax, pRect->Ymax);
    GL_LineDraw( pGL, pRect->Xmax, pRect->Ymin, pRect->Xmax, pRect->Ymax);
    pGL->cFore = cColor;

    return;
} 

STATIC VOID
Widget_DrawZip_Button(PGL pGL, WIDGET_PDRAW_BUTTON pDrawButton )
{
	//DBGLINE(DBG_LVL_UPGRADE,"Widget_DrawZip_Button start \n");
    RECT        Rect;
    RECT        RectFont;
    UINT32      cUp, cDn;

	//RectOffset( &pDrawButton->Rect, X, Y );
    Rect = pDrawButton->Rect;
    Rect.Ymax += 1;

    RectFont = Rect;
    RectInset( &RectFont, 1, 1);

    GL_SetForeColor( pGL, USBDL_COLOR_BUTTON );
    GL_RectFill( pGL, &Rect );

    if( pDrawButton->State & (1<<0) )
    {
        // INSET
        cUp = DK_GRAY;
        cDn = WHITE;
    }
    else
    {
        // OUTSET
        cUp = WHITE;
        cDn = DK_GRAY;
    }

    GL_SetForeColor( pGL, cUp);
    GL_SetBackColor( pGL, cDn);

    Rect3DDraw( pGL, &pDrawButton->Rect );

//DEBUGMSG(DBG_LVL_UPGRADE,"GT_DrawPushButton RectFont = %D\n",RectFont.Xmin);
//DEBUGMSG(DBG_LVL_UPGRADE,"GT_DrawPushButton RectFont = %D\n",RectFont.Ymin);
//DEBUGMSG(DBG_LVL_UPGRADE,"GT_DrawPushButton RectFont = %D\n",RectFont.Xmax);
//DEBUGMSG(DBG_LVL_UPGRADE,"GT_DrawPushButton RectFont = %D\n",RectFont.Ymax);

    // draw text
    if( pDrawButton->State & (1<<1) )
    {
        //disabled
        GL_SetForeColor( pGL, WT_GRAY );
    }
    else
    {
        GL_SetForeColor( pGL, BLACK );		
    }

    if( pDrawButton->State & (1<<0) )
    {
        RectFont.Xmin -= 2;
        RectFont.Ymin -= 2;
    }

    PSDI    pSDI;
	pSDI = GL_GetSDI(pGL);

	int nWeight = 6;
	int nValue = pSDI->Xres/100.0;
    FontSet( pGL, nWeight * nValue );

    GL_SetBackColor( pGL, LT_GRAY );
    GL_FontSetAlign( pGL, FONT_CENTER|FONT_VCENTER);
    GL_TextDraw( pGL, &RectFont, pDrawButton->pData );
	
	//DBGLINE(DBG_LVL_UPGRADE,"Widget_DrawZip_Button end\n");

    return;
} 

STATIC VOID Widget_DrawProgressBarZipPercent( PGL pGL, WIDGET_PDRAW_PROGRESSBAR pDrawProgressBar)
{
    //DBGLINE(DBG_LVL_UPGRADE, "Widget_DrawProgressBarZipPercent \n");
    RECT                Rect;
    BYTE                PerBuf[8];

    Rect = pDrawProgressBar->Rect;

    GL_SetForeColor( pGL, GRAY);
    GL_SetBackColor( pGL, WHITE);
    Rect3DDraw( pGL, &Rect);

    RectInset( &Rect, 1, 1);
    GL_SetForeColor( pGL, BLACK );
    GL_RectDraw( pGL, &Rect );
//DBGLINE(DBG_LVL_UPGRADE, "\n");
    RectInset( &Rect, 1, 1);
    GL_SetForeColor( pGL, LT_GRAY);
    GL_RectDraw( pGL, &Rect );
//DBGLINE(DBG_LVL_UPGRADE, "\n");
    RectInset( &Rect, 1, 1);

//	pDrawProgressBar->nPercent = downloadPercent * 100; // 100 -> 10000
DEBUGMSG(DBG_LVL_UPGRADE,"Widget_DrawProgressBar %X\n",pDrawProgressBar->nPercent );

    if(pDrawProgressBar->Style & PBS_VERTICAL)
    {
        UINT32              Ypos;

        Ypos = Rect.Ymin;
        Rect.Ymin = Rect.Ymax - (Rect.Ymax - Ypos)*pDrawProgressBar->nPercent/10000;
        //Rect.Ymin += Ypos;
        
        GL_SetForeColor( pGL, USBDL_COLOR_PROGRESSBAR );
        GL_RectFill( pGL, &Rect );
    //DBGLINE(DBG_LVL_UPGRADE, "\n");
        if( Ypos < Rect.Ymin)
        {
            Rect.Ymax = Rect.Ymin - 1;
            Rect.Ymin = Ypos;// Rect.Ymax + 1;
            GL_SetForeColor( pGL, USBDL_COLOR_PROGRESSBAR_BACK );
            GL_RectFill( pGL, &Rect );
        }
    }
    else
    {
        UINT32              Xpos;

        Xpos = Rect.Xmax;
        Rect.Xmax = (Xpos - Rect.Xmin)*pDrawProgressBar->nPercent/10000;
        Rect.Xmax += Rect.Xmin;
        
        GL_SetForeColor( pGL, USBDL_COLOR_PROGRESSBAR );
        GL_RectFill( pGL, &Rect );
   // DBGLINE(DBG_LVL_UPGRADE, "\n");
        Rect.Xmin = Rect.Xmax + 1;
        if( Xpos > Rect.Xmin)
        {
            Rect.Xmax = Xpos;

            GL_SetForeColor( pGL, USBDL_COLOR_PROGRESSBAR_BACK );
            GL_RectFill( pGL, &Rect );
           //DBGLINE(DBG_LVL_UPGRADE, "\n");
        }
    }

//    if( pDrawProgressBar->State ) // no info

    GL_FontSetAlign( pGL, FONT_CENTER|FONT_VCENTER);
    GL_SetBkMode(pGL, GL_TRANSPARENCY_TRANSPARENT);
    //GL_SetDrawMode( pGL, ROP_XOR);
    GL_SetForeColor( pGL, WHITE );
    sprintf( (char *)PerBuf, "%d%%", pDrawProgressBar->nPercent/100);
    Rect = pDrawProgressBar->Rect;
    //GL_TextDrawCall( pGL, &Rect, PerBuf );
    GL_TextDraw( pGL, &Rect, PerBuf );
    //DBGLINE(DBG_LVL_UPGRADE, "\n");
    //GL_SetDrawMode( pGL, ROP_COPY);

    return;
}  

PUBLIC VOID func_ZipWnd( INT32 option )
{
    MbUpgradeSetting.option = option;
    
    DBGLINE(DBG_LVL_UPGRADE, "hWndZipUpgrade=%x\n", hWndZipUpgrade);
    if(hWndZipUpgrade) ShowWindow(hWndZipUpgrade, SW_SHOW);    
}
